/**
* @file Sort.java
*
* @brief This file contains the abstract sorting.Sort class.
*
* @author Philipp Bank, www.cypax.net\n
*
* @date June, 18th 2006 - first version
*
**/

package sorting;

import java.awt.*;

/**
* @class Sort
* @brief The Sort class is the super class for all sorting classes in this package.
*
* This class provides some functions and attributes, which are used by all child classes.
* For example, each sorting algorithm in this package uses a exchange() function to change the positions of two array elements.
* So this function is implemented once in this super class and then inherited to each child class.
* But this super class provides only an empty sorting function - this function must be overloaded by child classes with
* different sorting algorithms (therefore it makes no sense to create instances of this super class and therfore this is an abstract class).
*/
public abstract class Sort implements Runnable
{
  protected long startTime = 0;
  protected long endTime = 0;
  protected long runTime = 0;

  protected int[] theArray;
  protected Panel thePanel;
  protected Label theLabel;

/**
* @brief This function is automatically executet when starting the thread.
*
* The run() function calls the doSort() function, which is overloaded by child classes.
*/
  public void run()
  {
    this.startTime = System.currentTimeMillis();
    doSort();   // call the sorting function - this function must be overloaded by child classes
    this.showArray();
    this.theLabel.setText(this.calcRuntime()+"ms - done!");
    this.theLabel.validate();
  }

/**
* @brief This function returns the integer array theArray
*
* This function is used to get access to the sortet integer numbers.
*
* @return - returns the integer array theArray
*/
  public int[] getSortedArray()
  {
    return this.theArray;
  }

/**
* @brief This is just a empty sorting function to be overloaded by child classes.
*
* Child classes overload this function with own doSort() functions and different algorithms.
*/
  protected void doSort()
  {
  }


/**
* @brief This function changes the positions of two array elements.
*
* @param aArray - an integer array
* @param index1 - the 1st position
* @param index2 - the 2nd position
*/
  protected void exchange(int[] aArray, int index1, int index2)
  {
    int tmp = aArray[index1];
    aArray[index1] = aArray[index2];
    aArray[index2] = tmp;
  }

/**
* @brief This function calculates the runtime of the thread
*
* @return - returns the time, since thread has been started in milliseconds
*/
  protected long calcRuntime()
  {
    return (System.currentTimeMillis()-startTime);
  }

/**
* @brief This function is used to visualize the integer numbers of the integer array by painting the numbers as vertical green lines.
*
* @note: This function requires, that the setSettings() function has been called previously.
*/
  public void showArray()
  {
    Graphics g = thePanel.getGraphics();
    Color lc = new Color(0,255,0);
    Color bc = new Color(128,128,128);
    for (int i=0; i<210; i++)
    {
      g.setColor(lc);
      g.drawLine((i+20),190,(i+20),(190-theArray[i]));
      g.setColor(bc);
      g.drawLine((i+20),(190-theArray[i]),(i+20),10);
    }
    thePanel.validate();
  }

/**
* @brief This function is used to set the attributes theArray, thePanel and theLabel
*
* @param aArray - an integer array to be sorted
* @param aPanel - a panel, where graphical visualition of the array shall occour during sorting
* @param aLabel - a label, where to show the running time
*/
  public void setSettings(int[] aArray, Panel aPanel, Label aLabel)
  {
    this.theArray = aArray;
    this.thePanel = aPanel;
    this.theLabel = aLabel;
  }
  
}

